@@ -1,7 +1,7 @@
 /*
  *  be_package.c : backend for packages
  *
- *  Copyright (c) 2006-2013 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2014 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -39,6 +39,7 @@
 #include "package.h"
 #include "deps.h"
 #include "filelist.h"
+#include "util.h"
 
 struct package_changelog {
 	struct archive *archive;
@@ -76,7 +77,7 @@ static void *_package_changelog_open(alpm_pkg_t *pkg)
 			if(!changelog) {
 				pkg->handle->pm_errno = ALPM_ERR_MEMORY;
 				_alpm_archive_read_free(archive);
-				CLOSE(fd);
+				close(fd);
 				return NULL;
 			}
 			changelog->archive = archive;
@@ -86,7 +87,7 @@ static void *_package_changelog_open(alpm_pkg_t *pkg)
 	}
 	/* we didn't find a changelog */
 	_alpm_archive_read_free(archive);
-	CLOSE(fd);
+	close(fd);
 	errno = ENOENT;
 
 	return NULL;
@@ -126,7 +127,7 @@ static int _package_changelog_close(const alpm_pkg_t UNUSED *pkg, void *fp)
 	int ret;
 	struct package_changelog *changelog = fp;
 	ret = _alpm_archive_read_free(changelog->archive);
-	CLOSE(changelog->fd);
+	close(changelog->fd);
 	free(changelog);
 	return ret;
 }
@@ -195,6 +196,8 @@ static int parse_descfile(alpm_handle_t *handle, struct archive *a, alpm_pkg_t *
 				/* not used atm */
 			} else if(strcmp(key, "pkgver") == 0) {
 				STRDUP(newpkg->version, ptr, return -1);
+			} else if(strcmp(key, "basever") == 0) {
+				/* not used atm */
 			} else if(strcmp(key, "pkgdesc") == 0) {
 				STRDUP(newpkg->desc, ptr, return -1);
 			} else if(strcmp(key, "group") == 0) {
@@ -213,19 +216,23 @@ static int parse_descfile(alpm_handle_t *handle, struct archive *a, alpm_pkg_t *
 				/* size in the raw package is uncompressed (installed) size */
 				newpkg->isize = _alpm_strtoofft(ptr);
 			} else if(strcmp(key, "depend") == 0) {
-				alpm_depend_t *dep = _alpm_splitdep(ptr);
+				alpm_depend_t *dep = alpm_dep_from_string(ptr);
 				newpkg->depends = alpm_list_add(newpkg->depends, dep);
 			} else if(strcmp(key, "optdepend") == 0) {
-				alpm_depend_t *optdep = _alpm_splitdep(ptr);
+				alpm_depend_t *optdep = alpm_dep_from_string(ptr);
 				newpkg->optdepends = alpm_list_add(newpkg->optdepends, optdep);
+			} else if(strcmp(key, "makedepend") == 0) {
+				/* not used atm */
+			} else if(strcmp(key, "checkdepend") == 0) {
+				/* not used atm */
 			} else if(strcmp(key, "conflict") == 0) {
-				alpm_depend_t *conflict = _alpm_splitdep(ptr);
+				alpm_depend_t *conflict = alpm_dep_from_string(ptr);
 				newpkg->conflicts = alpm_list_add(newpkg->conflicts, conflict);
 			} else if(strcmp(key, "replaces") == 0) {
-				alpm_depend_t *replace = _alpm_splitdep(ptr);
+				alpm_depend_t *replace = alpm_dep_from_string(ptr);
 				newpkg->replaces = alpm_list_add(newpkg->replaces, replace);
 			} else if(strcmp(key, "provides") == 0) {
-				alpm_depend_t *provide = _alpm_splitdep(ptr);
+				alpm_depend_t *provide = alpm_dep_from_string(ptr);
 				newpkg->provides = alpm_list_add(newpkg->provides, provide);
 			} else if(strcmp(key, "backup") == 0) {
 				alpm_backup_t *backup;
@@ -323,9 +330,13 @@ int _alpm_pkg_validate_internal(alpm_handle_t *handle,
 	}
 
 	/* even if we don't have a sig, run the check code if level tells us to */
-	if(has_sig || level & ALPM_SIG_PACKAGE) {
+	if(level & ALPM_SIG_PACKAGE) {
 		const char *sig = syncpkg ? syncpkg->base64_sig : NULL;
 		_alpm_log(handle, ALPM_LOG_DEBUG, "sig data: %s\n", sig ? sig : "<from .sig>");
+		if(!has_sig && !(level & ALPM_SIG_PACKAGE_OPTIONAL)) {
+			handle->pm_errno = ALPM_ERR_PKG_MISSING_SIG;
+			return -1;
+		}
 		if(_alpm_check_pgp_helper(handle, pkgfile, sig,
 					level & ALPM_SIG_PACKAGE_OPTIONAL, level & ALPM_SIG_PACKAGE_MARGINAL_OK,
 					level & ALPM_SIG_PACKAGE_UNKNOWN_OK, sigdata)) {
@@ -344,6 +355,181 @@ int _alpm_pkg_validate_internal(alpm_handle_t *handle,
 	return 0;
 }
 
+/**
+ * Handle the existance of simple paths for _alpm_load_pkg_internal()
+ * @param pkg package to change
+ * @param path path to examine
+ * @return 0 if path doesn't match any rule, 1 if it has been handled
+ */
+static int handle_simple_path(alpm_pkg_t *pkg, const char *path)
+{
+	if(strcmp(path, ".INSTALL") == 0) {
+		pkg->scriptlet = 1;
+		return 1;
+	} else if(*path == '.') {
+		/* for now, ignore all files starting with '.' that haven't
+		 * already been handled (for future possibilities) */
+		return 1;
+	}
+
+	return 0;
+}
+
+/**
+ * Add a file to the files list for pkg.
+ *
+ * @param pkg package to add the file to
+ * @param files_size size of pkg->files.files
+ * @param entry archive entry of the file to add to the list
+ * @param path path of the file to be added
+ * @return <0 on error, 0 on success
+ */
+static int add_entry_to_files_list(alpm_pkg_t *pkg, size_t *files_size,
+		struct archive_entry *entry, const char *path)
+{
+	const size_t files_count = pkg->files.count;
+	alpm_file_t *current_file;
+	mode_t type;
+	size_t pathlen;
+
+	if(!_alpm_greedy_grow((void **)&pkg->files.files, files_size, (files_count + 1) * sizeof(alpm_file_t))) {
+		return -1;
+	}
+
+	type = archive_entry_filetype(entry);
+
+	pathlen = strlen(path);
+
+	current_file = pkg->files.files + files_count;
+
+	/* mtree paths don't contain a tailing slash, those we get from
+	 * the archive directly do (expensive way)
+	 * Other code relies on it to detect directories so add it here.*/
+	if(type == AE_IFDIR && path[pathlen - 1] != '/') {
+		/* 2 = 1 for / + 1 for \0 */
+		char *newpath;
+		MALLOC(newpath, pathlen + 2, return -1);
+		strcpy(newpath, path);
+		newpath[pathlen] = '/';
+		newpath[pathlen + 1] = '\0';
+		current_file->name = newpath;
+	} else {
+		STRDUP(current_file->name, path, return -1);
+	}
+	current_file->size = archive_entry_size(entry);
+	current_file->mode = archive_entry_mode(entry);
+	pkg->files.count++;
+	return 0;
+}
+
+/**
+ * Generate a new file list from an mtree file and add it to the package.
+ * An existing file list will be free()d first.
+ *
+ * archive should point to an archive struct which is already at the
+ * position of the mtree's header.
+ *
+ * @param handle
+ * @param pkg package to add the file list to
+ * @param archive archive containing the mtree
+ * @return 0 on success, <0 on error
+ */
+static int build_filelist_from_mtree(alpm_handle_t *handle, alpm_pkg_t *pkg, struct archive *archive)
+{
+	int ret = 0;
+	size_t mtree_maxsize = 0;
+	size_t mtree_cursize = 0;
+	size_t files_size = 0; /* we clean up the existing array so this is fine */
+	char *mtree_data = NULL;
+	struct archive *mtree;
+	struct archive_entry *mtree_entry = NULL;
+
+	_alpm_log(handle, ALPM_LOG_DEBUG,
+			"found mtree for package %s, getting file list\n", pkg->filename);
+
+	/* throw away any files we might have already found */
+	for (size_t i = 0; i < pkg->files.count; i++) {
+		free(pkg->files.files[i].name);
+	}
+	free(pkg->files.files);
+	pkg->files.files = NULL;
+	pkg->files.count = 0;
+
+	/* create a new archive to parse the mtree and load it from archive into memory */
+	/* TODO: split this into a function */
+	if((mtree = archive_read_new()) == NULL) {
+		handle->pm_errno = ALPM_ERR_LIBARCHIVE;
+		goto error;
+	}
+
+	_alpm_archive_read_support_filter_all(mtree);
+	archive_read_support_format_mtree(mtree);
+
+	/* TODO: split this into a function */
+	while(1) {
+		ssize_t size;
+
+		if(!_alpm_greedy_grow((void **)&mtree_data, &mtree_maxsize, mtree_cursize + ALPM_BUFFER_SIZE)) {
+			goto error;
+		}
+
+		size = archive_read_data(archive, mtree_data + mtree_cursize, ALPM_BUFFER_SIZE);
+
+		if(size < 0) {
+			_alpm_log(handle, ALPM_LOG_ERROR, _("error while reading package %s: %s\n"),
+					pkg->filename, archive_error_string(archive));
+			handle->pm_errno = ALPM_ERR_LIBARCHIVE;
+			goto error;
+		}
+		if(size == 0) {
+			break;
+		}
+
+		mtree_cursize += size;
+	}
+
+	if(archive_read_open_memory(mtree, mtree_data, mtree_cursize)) {
+		_alpm_log(handle, ALPM_LOG_ERROR,
+				_("error while reading mtree of package %s: %s\n"),
+				pkg->filename, archive_error_string(mtree));
+		handle->pm_errno = ALPM_ERR_LIBARCHIVE;
+		goto error;
+	}
+
+	while((ret = archive_read_next_header(mtree, &mtree_entry)) == ARCHIVE_OK) {
+		const char *path = archive_entry_pathname(mtree_entry);
+
+		/* strip leading "./" from path entries */
+		if(path[0] == '.' && path[1] == '/') {
+			path += 2;
+		}
+
+		if(handle_simple_path(pkg, path)) {
+			continue;
+		}
+
+		if(add_entry_to_files_list(pkg, &files_size, mtree_entry, path) < 0) {
+			goto error;
+		}
+	}
+
+	if(ret != ARCHIVE_EOF && ret != ARCHIVE_OK) { /* An error occurred */
+		_alpm_log(handle, ALPM_LOG_ERROR, _("error while reading mtree of package %s: %s\n"),
+				pkg->filename, archive_error_string(mtree));
+		handle->pm_errno = ALPM_ERR_LIBARCHIVE;
+		goto error;
+	}
+
+	free(mtree_data);
+	_alpm_archive_read_free(mtree);
+	_alpm_log(handle, ALPM_LOG_DEBUG, "finished mtree reading for %s\n", pkg->filename);
+	return 0;
+error:
+	free(mtree_data);
+	_alpm_archive_read_free(mtree);
+	return -1;
+}
+
 /**
  * Load a package and create the corresponding alpm_pkg_t struct.
  * @param handle the context handle
@@ -354,7 +540,9 @@ int _alpm_pkg_validate_internal(alpm_handle_t *handle,
 alpm_pkg_t *_alpm_pkg_load_internal(alpm_handle_t *handle,
 		const char *pkgfile, int full)
 {
-	int ret, fd, config = 0;
+	int ret, fd;
+	int config = 0;
+	int hit_mtree = 0;
 	struct archive *archive;
 	struct archive_entry *entry;
 	alpm_pkg_t *newpkg;
@@ -409,42 +597,28 @@ alpm_pkg_t *_alpm_pkg_load_internal(alpm_handle_t *handle,
 				_alpm_log(handle, ALPM_LOG_ERROR, _("missing package version in %s\n"), pkgfile);
 				goto pkg_invalid;
 			}
+			if(strchr(newpkg->version, '-') == NULL) {
+				_alpm_log(handle, ALPM_LOG_ERROR, _("invalid package version in %s\n"), pkgfile);
+				goto pkg_invalid;
+			}
 			config = 1;
 			continue;
-		} else if(strcmp(entry_name,  ".INSTALL") == 0) {
-			newpkg->scriptlet = 1;
-		} else if(*entry_name == '.') {
-			/* for now, ignore all files starting with '.' that haven't
-			 * already been handled (for future possibilities) */
-		} else if(full) {
-			const size_t files_count = newpkg->files.count;
-			alpm_file_t *current_file;
-			/* Keep track of all files for filelist generation */
-			if(files_count >= files_size) {
-				size_t old_size = files_size;
-				alpm_file_t *newfiles;
-				if(files_size == 0) {
-					files_size = 4;
-				} else {
-					files_size *= 2;
-				}
-				newfiles = realloc(newpkg->files.files,
-						sizeof(alpm_file_t) * files_size);
-				if(!newfiles) {
-					_alpm_alloc_fail(sizeof(alpm_file_t) * files_size);
-					goto error;
-				}
-				/* ensure all new memory is zeroed out, in both the initial
-				 * allocation and later reallocs */
-				memset(newfiles + old_size, 0,
-						sizeof(alpm_file_t) * (files_size - old_size));
-				newpkg->files.files = newfiles;
+		} else if(full && strcmp(entry_name, ".MTREE") == 0) {
+			/* building the file list: cheap way
+			 * get the filelist from the mtree file rather than scanning
+			 * the whole archive  */
+			if(build_filelist_from_mtree(handle, newpkg, archive) < 0) {
+				goto error;
+			}
+			hit_mtree = 1;
+			continue;
+		} else if(handle_simple_path(newpkg, entry_name)) {
+			continue;
+		} else if(full && !hit_mtree) {
+			/* building the file list: expensive way */
+			if(add_entry_to_files_list(newpkg, &files_size, entry, entry_name) < 0) {
+				goto error;
 			}
-			current_file = newpkg->files.files + files_count;
-			STRDUP(current_file->name, entry_name, goto error);
-			current_file->size = archive_entry_size(entry);
-			current_file->mode = archive_entry_mode(entry);
-			newpkg->files.count++;
 		}
 
 		if(archive_read_data_skip(archive)) {
@@ -455,7 +629,7 @@ alpm_pkg_t *_alpm_pkg_load_internal(alpm_handle_t *handle,
 		}
 
 		/* if we are not doing a full read, see if we have all we need */
-		if(!full && config) {
+		if((!full || hit_mtree) && config) {
 			break;
 		}
 	}
@@ -473,7 +647,7 @@ alpm_pkg_t *_alpm_pkg_load_internal(alpm_handle_t *handle,
 	}
 
 	_alpm_archive_read_free(archive);
-	CLOSE(fd);
+	close(fd);
 
 	/* internal fields for package struct */
 	newpkg->origin = ALPM_PKG_FROM_FILE;
@@ -506,7 +680,7 @@ error:
 	_alpm_pkg_free(newpkg);
 	_alpm_archive_read_free(archive);
 	if(fd >= 0) {
-		CLOSE(fd);
+		close(fd);
 	}
 
 	return NULL;
@@ -562,7 +736,7 @@ int SYMEXPORT alpm_pkg_load(alpm_handle_t *handle, const char *filename, int ful
 				return -1;
 			}
 
-			if(_alpm_extract_keyid(handle, filename, sig, len, &keys) == 0) {
+			if(alpm_extract_keyid(handle, filename, sig, len, &keys) == 0) {
 				alpm_list_t *k;
 				for(k = keys; k; k = k->next) {
 					char *key = k->data;
@@ -600,4 +774,4 @@ int SYMEXPORT alpm_pkg_load(alpm_handle_t *handle, const char *filename, int ful
 	return 0;
 }
 
-/* vim: set ts=2 sw=2 noet: */
+/* vim: set noet: */
