@@ -1,7 +1,7 @@
 /*
  *  diskspace.h
  *
- *  Copyright (c) 2010-2011 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2010-2014 Pacman Development Team <pacman-dev@archlinux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -37,6 +37,12 @@ enum mount_used_level {
 	USED_INSTALL = (1 << 1),
 };
 
+enum mount_fsinfo {
+	MOUNT_FSINFO_UNLOADED = 0,
+	MOUNT_FSINFO_LOADED,
+	MOUNT_FSINFO_FAIL,
+};
+
 typedef struct __alpm_mountpoint_t {
 	/* mount point information */
 	char *mount_dir;
@@ -46,11 +52,14 @@ typedef struct __alpm_mountpoint_t {
 	blkcnt_t max_blocks_needed;
 	enum mount_used_level used;
 	int read_only;
+	enum mount_fsinfo fsinfo_loaded;
 	FSSTATSTYPE fsp;
 } alpm_mountpoint_t;
 
 int _alpm_check_diskspace(alpm_handle_t *handle);
+int _alpm_check_downloadspace(alpm_handle_t *handle, const char *cachedir,
+		size_t num_files, off_t *file_sizes);
 
 #endif /* _ALPM_DISKSPACE_H */
 
-/* vim: set ts=2 sw=2 noet: */
+/* vim: set noet: */
