@@ -1,7 +1,7 @@
 /*
  *  graph.h - helpful graph structure and setup/teardown methods
  *
- *  Copyright (c) 2007-2011 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2007-2014 Pacman Development Team <pacman-dev@archlinux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,19 +19,17 @@
 #ifndef _ALPM_GRAPH_H
 #define _ALPM_GRAPH_H
 
-#include "config.h" /* ensure off_t is correct length */
-
 #include <sys/types.h> /* off_t */
 
 #include "alpm_list.h"
 
 typedef struct __alpm_graph_t {
-	char state; /* 0: untouched, -1: entered, other: leaving time */
-	off_t weight; /* weight of the node */
 	void *data;
 	struct __alpm_graph_t *parent; /* where did we come from? */
 	alpm_list_t *children;
 	alpm_list_t *childptr; /* points to a child in children list */
+	off_t weight; /* weight of the node */
+	char state; /* 0: untouched, -1: entered, other: leaving time */
 } alpm_graph_t;
 
 alpm_graph_t *_alpm_graph_new(void);
@@ -39,4 +37,4 @@ void _alpm_graph_free(void *data);
 
 #endif /* _ALPM_GRAPH_H */
 
-/* vim: set ts=2 sw=2 noet: */
+/* vim: set noet: */
