@@ -1,7 +1,7 @@
 /*
  *  log.h
  *
- *  Copyright (c) 2006-2011 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2014 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -22,9 +22,11 @@
 
 #include "alpm.h"
 
+#define ALPM_CALLER_PREFIX "ALPM"
+
 void _alpm_log(alpm_handle_t *handle, alpm_loglevel_t flag,
 		const char *fmt, ...) __attribute__((format(printf,3,4)));
 
 #endif /* _ALPM_LOG_H */
 
-/* vim: set ts=2 sw=2 noet: */
+/* vim: set noet: */
