@@ -1,7 +1,7 @@
 /*
  *  trans.c
  *
- *  Copyright (c) 2006-2011 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2014 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *  Copyright (c) 2005 by Aurelien Foret <orelien@chez.com>
  *  Copyright (c) 2005 by Christian Hamar <krics@linuxforum.hu>
@@ -21,8 +21,6 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "config.h"
-
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -41,6 +39,7 @@
 #include "remove.h"
 #include "sync.h"
 #include "alpm.h"
+#include "deps.h"
 
 /** \addtogroup alpm_trans Transaction Functions
  * @brief Functions to manipulate libalpm transactions
@@ -136,6 +135,21 @@ int SYMEXPORT alpm_trans_prepare(alpm_handle_t *handle, alpm_list_t **data)
 		}
 	}
 
+
+	if(!(trans->flags & ALPM_TRANS_FLAG_NODEPS)) {
+		_alpm_log(handle, ALPM_LOG_DEBUG, "sorting by dependencies\n");
+		if(trans->add) {
+			alpm_list_t *add_orig = trans->add;
+			trans->add = _alpm_sortbydeps(handle, add_orig, trans->remove, 0);
+			alpm_list_free(add_orig);
+		}
+		if(trans->remove) {
+			alpm_list_t *rem_orig = trans->remove;
+			trans->remove = _alpm_sortbydeps(handle, rem_orig, NULL, 1);
+			alpm_list_free(rem_orig);
+		}
+	}
+
 	trans->state = STATE_PREPARED;
 
 	return 0;
@@ -161,20 +175,40 @@ int SYMEXPORT alpm_trans_commit(alpm_handle_t *handle, alpm_list_t **data)
 		return 0;
 	}
 
+	if(trans->add) {
+		if(_alpm_sync_load(handle, data) != 0) {
+			/* pm_errno is set by _alpm_sync_load() */
+			return -1;
+		}
+		if(trans->flags & ALPM_TRANS_FLAG_DOWNLOADONLY) {
+			return 0;
+		}
+	}
+
 	trans->state = STATE_COMMITING;
 
+	alpm_logaction(handle, ALPM_CALLER_PREFIX, "transaction started\n");
+
 	if(trans->add == NULL) {
 		if(_alpm_remove_packages(handle, 1) == -1) {
-			/* pm_errno is set by _alpm_remove_commit() */
+			/* pm_errno is set by _alpm_remove_packages() */
+			alpm_logaction(handle, ALPM_CALLER_PREFIX, "transaction failed\n");
 			return -1;
 		}
 	} else {
 		if(_alpm_sync_commit(handle, data) == -1) {
 			/* pm_errno is set by _alpm_sync_commit() */
+			alpm_logaction(handle, ALPM_CALLER_PREFIX, "transaction failed\n");
 			return -1;
 		}
 	}
 
+	if(trans->state == STATE_INTERRUPTED) {
+		alpm_logaction(handle, ALPM_CALLER_PREFIX, "transaction interrupted\n");
+	} else {
+		alpm_logaction(handle, ALPM_CALLER_PREFIX, "transaction completed\n");
+	}
+
 	trans->state = STATE_COMMITED;
 
 	return 0;
@@ -217,12 +251,7 @@ int SYMEXPORT alpm_trans_release(alpm_handle_t *handle)
 
 	/* unlock db */
 	if(!nolock_flag) {
-		if(_alpm_handle_unlock(handle)) {
-			_alpm_log(handle, ALPM_LOG_WARNING, _("could not remove lock file %s\n"),
-					handle->lockfile);
-			alpm_logaction(handle, "warning: could not remove lock file %s\n",
-					handle->lockfile);
-		}
+		_alpm_handle_unlock(handle);
 	}
 
 	return 0;
@@ -261,7 +290,7 @@ static int grep(const char *fn, const char *needle)
 	}
 	while(!feof(fp)) {
 		char line[1024];
-		if(fgets(line, sizeof(line), fp) == NULL) {
+		if(safe_fgets(line, sizeof(line), fp) == NULL) {
 			continue;
 		}
 		/* TODO: this will not work if the search string
@@ -278,8 +307,8 @@ static int grep(const char *fn, const char *needle)
 int _alpm_runscriptlet(alpm_handle_t *handle, const char *filepath,
 		const char *script, const char *ver, const char *oldver, int is_archive)
 {
-	char cmdline[PATH_MAX];
-	char *argv[] = { "sh", "-c", cmdline, NULL };
+	char arg0[64], arg1[3], cmdline[PATH_MAX];
+	char *argv[] = { arg0, arg1, cmdline, NULL };
 	char *tmpdir, *scriptfn = NULL, *scriptpath;
 	int retval = 0;
 	size_t len;
@@ -291,10 +320,13 @@ int _alpm_runscriptlet(alpm_handle_t *handle, const char *filepath,
 
 	if(!is_archive && !grep(filepath, script)) {
 		/* script not found in scriptlet file; we can only short-circuit this early
-		 * if it is an actual scriptlet file and not an archive.  */
+		 * if it is an actual scriptlet file and not an archive. */
 		return 0;
 	}
 
+	strcpy(arg0, SCRIPTLET_SHELL);
+	strcpy(arg1, "-c");
+
 	/* create a directory in $root/tmp/ for copying/extracting the scriptlet */
 	len = strlen(handle->root) + strlen("tmp/alpm_XXXXXX") + 1;
 	MALLOC(tmpdir, len, RET_ERR(handle, ALPM_ERR_MEMORY, -1));
@@ -345,7 +377,7 @@ int _alpm_runscriptlet(alpm_handle_t *handle, const char *filepath,
 
 	_alpm_log(handle, ALPM_LOG_DEBUG, "executing \"%s\"\n", cmdline);
 
-	retval = _alpm_run_chroot(handle, "/bin/sh", argv);
+	retval = _alpm_run_chroot(handle, SCRIPTLET_SHELL, argv);
 
 cleanup:
 	if(scriptfn && unlink(scriptfn)) {
@@ -388,4 +420,4 @@ alpm_list_t SYMEXPORT *alpm_trans_get_remove(alpm_handle_t *handle)
 
 	return handle->trans->remove;
 }
-/* vim: set ts=2 sw=2 noet: */
+/* vim: set noet: */
