@@ -1,7 +1,7 @@
 /*
  *  callback.h
  *
- *  Copyright (c) 2006-2011 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2014 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -24,15 +24,14 @@
 
 #include <alpm.h>
 
-/* callback to handle messages/notifications from libalpm transactions */
-void cb_trans_evt(pmtransevt_t event, void *data1, void *data2);
+/* callback to handle messages/notifications from libalpm */
+void cb_event(alpm_event_t *event);
 
-/* callback to handle questions from libalpm transactions (yes/no) */
-void cb_trans_conv(pmtransconv_t event, void *data1, void *data2,
-                   void *data3, int *response);
+/* callback to handle questions from libalpm (yes/no) */
+void cb_question(alpm_question_t *question);
 
-/* callback to handle display of transaction progress */
-void cb_trans_progress(pmtransprog_t event, const char *pkgname, int percent,
+/* callback to handle display of progress */
+void cb_progress(alpm_progress_t event, const char *pkgname, int percent,
                    size_t howmany, size_t remain);
 
 /* callback to handle receipt of total download value */
@@ -41,8 +40,9 @@ void cb_dl_total(off_t total);
 void cb_dl_progress(const char *filename, off_t file_xfered, off_t file_total);
 
 /* callback to handle messages/notifications from pacman library */
-void cb_log(pmloglevel_t level, const char *fmt, va_list args);
+__attribute__((format(printf, 2, 0)))
+void cb_log(alpm_loglevel_t level, const char *fmt, va_list args);
 
 #endif /* _PM_CALLBACK_H */
 
-/* vim: set ts=2 sw=2 noet: */
+/* vim: set noet: */
