@@ -1,6 +1,7 @@
 /*
  *  testdb.c : Test a pacman local database for validity
  *
+ *  Copyright (c) 2007-2014 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2007 by Aaron Griffin <aaronmgriffin@gmail.com>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -17,8 +18,6 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "config.h"
-
 #include <unistd.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -29,8 +28,6 @@
 #include <alpm.h>
 #include <alpm_list.h>
 
-#define BASENAME "testdb"
-
 alpm_handle_t *handle = NULL;
 
 static void cleanup(int signum)
@@ -42,6 +39,7 @@ static void cleanup(int signum)
 	exit(signum);
 }
 
+__attribute__((format(printf, 2, 0)))
 static void output_cb(alpm_loglevel_t level, const char *fmt, va_list args)
 {
 	if(strlen(fmt)) {
@@ -65,24 +63,24 @@ static int check_localdb_files(void)
 	dbpath = alpm_option_get_dbpath(handle);
 	snprintf(path, sizeof(path), "%slocal", dbpath);
 	if(!(dir = opendir(path))) {
-		fprintf(stderr, "error : %s : %s\n", path, strerror(errno));
+		fprintf(stderr, "error : '%s' : %s\n", path, strerror(errno));
 		return 1;
 	}
 
 	while((ent = readdir(dir)) != NULL) {
 		if(strcmp(ent->d_name, ".") == 0 || strcmp(ent->d_name, "..") == 0
-				|| ent->d_name[0] == '.') {
+				|| strcmp(ent->d_name, "ALPM_DB_VERSION") == 0) {
 			continue;
 		}
 		/* check for known db files in local database */
 		snprintf(path, sizeof(path), "%slocal/%s/desc", dbpath, ent->d_name);
 		if(access(path, F_OK)) {
-			printf("%s: description file is missing\n", ent->d_name);
+			printf("'%s': description file is missing\n", ent->d_name);
 			ret++;
 		}
 		snprintf(path, sizeof(path), "%slocal/%s/files", dbpath, ent->d_name);
 		if(access(path, F_OK)) {
-			printf("%s: file list is missing\n", ent->d_name);
+			printf("'%s': file list is missing\n", ent->d_name);
 			ret++;
 		}
 	}
@@ -94,17 +92,16 @@ static int check_localdb_files(void)
 	return ret;
 }
 
-static int checkdeps(alpm_list_t *pkglist)
+static int check_deps(alpm_list_t *pkglist)
 {
 	alpm_list_t *data, *i;
 	int ret = 0;
 	/* check dependencies */
-	data = alpm_checkdeps(handle, pkglist, NULL, pkglist, 0);
+	data = alpm_checkdeps(handle, NULL, NULL, pkglist, 0);
 	for(i = data; i; i = alpm_list_next(i)) {
-		alpm_depmissing_t *miss = alpm_list_getdata(i);
+		alpm_depmissing_t *miss = i->data;
 		char *depstring = alpm_dep_compute_string(miss->depend);
-		printf("missing dependency for %s : %s\n", miss->target,
-				depstring);
+		printf("missing '%s' dependency for '%s'\n", depstring, miss->target);
 		free(depstring);
 		ret++;
 	}
@@ -112,15 +109,15 @@ static int checkdeps(alpm_list_t *pkglist)
 	return ret;
 }
 
-static int checkconflicts(alpm_list_t *pkglist)
+static int check_conflicts(alpm_list_t *pkglist)
 {
 	alpm_list_t *data, *i;
 	int ret = 0;
 	/* check conflicts */
 	data = alpm_checkconflicts(handle, pkglist);
 	for(i = data; i; i = i->next) {
-		alpm_conflict_t *conflict = alpm_list_getdata(i);
-		printf("%s conflicts with %s\n",
+		alpm_conflict_t *conflict = i->data;
+		printf("'%s' conflicts with '%s'\n",
 				conflict->package1, conflict->package2);
 		ret++;
 	}
@@ -128,6 +125,77 @@ static int checkconflicts(alpm_list_t *pkglist)
 	return ret;
 }
 
+struct fileitem {
+	alpm_file_t *file;
+	alpm_pkg_t *pkg;
+};
+
+static int fileitem_cmp(const void *p1, const void *p2)
+{
+	const struct fileitem * fi1 = p1;
+	const struct fileitem * fi2 = p2;
+	return strcmp(fi1->file->name, fi2->file->name);
+}
+
+static int check_filelists(alpm_list_t *pkglist)
+{
+	alpm_list_t *i;
+	int ret = 0;
+	size_t list_size = 4096;
+	size_t offset = 0, j;
+	struct fileitem *all_files;
+	struct fileitem *prev_fileitem = NULL;
+
+	all_files = malloc(list_size * sizeof(struct fileitem));
+
+	for(i = pkglist; i; i = i->next) {
+		alpm_pkg_t *pkg = i->data;
+		alpm_filelist_t *filelist = alpm_pkg_get_files(pkg);
+		for(j = 0; j < filelist->count; j++) {
+			alpm_file_t *file = filelist->files + j;
+			/* only add files, not directories, to our big list */
+			if(file->name[strlen(file->name) - 1] == '/') {
+				continue;
+			}
+
+			/* do we need to reallocate and grow our array? */
+			if(offset >= list_size) {
+				struct fileitem *new_files;
+				new_files = realloc(all_files, list_size * 2 * sizeof(struct fileitem));
+				if(!new_files) {
+					free(all_files);
+					return 1;
+				}
+				all_files = new_files;
+				list_size *= 2;
+			}
+
+			/* we can finally add it to the list */
+			all_files[offset].file = file;
+			all_files[offset].pkg = pkg;
+			offset++;
+		}
+	}
+
+	/* now sort the list so we can find duplicates */
+	qsort(all_files, offset, sizeof(struct fileitem), fileitem_cmp);
+
+	/* do a 'uniq' style check on the list */
+	for(j = 0; j < offset; j++) {
+		struct fileitem *fileitem = all_files + j;
+		if(prev_fileitem && fileitem_cmp(prev_fileitem, fileitem) == 0) {
+			printf("file owned by '%s' and '%s': '%s'\n",
+					alpm_pkg_get_name(prev_fileitem->pkg),
+					alpm_pkg_get_name(fileitem->pkg),
+					fileitem->file->name);
+		}
+		prev_fileitem = fileitem;
+	}
+
+	free(all_files);
+	return ret;
+}
+
 static int check_localdb(void)
 {
 	int ret = 0;
@@ -139,10 +207,11 @@ static int check_localdb(void)
 		return ret;
 	}
 
-	db = alpm_option_get_localdb(handle);
+	db = alpm_get_localdb(handle);
 	pkglist = alpm_db_get_pkgcache(db);
-	ret += checkdeps(pkglist);
-	ret += checkconflicts(pkglist);
+	ret += check_deps(pkglist);
+	ret += check_conflicts(pkglist);
+	ret += check_filelists(pkglist);
 	return ret;
 }
 
@@ -154,8 +223,8 @@ static int check_syncdbs(alpm_list_t *dbnames)
 	const alpm_siglevel_t level = ALPM_SIG_DATABASE | ALPM_SIG_DATABASE_OPTIONAL;
 
 	for(i = dbnames; i; i = alpm_list_next(i)) {
-		char *dbname = alpm_list_getdata(i);
-		db = alpm_db_register_sync(handle, dbname, level);
+		const char *dbname = i->data;
+		db = alpm_register_syncdb(handle, dbname, level);
 		if(db == NULL) {
 			fprintf(stderr, "error: could not register sync database (%s)\n",
 					alpm_strerror(alpm_errno(handle)));
@@ -165,7 +234,7 @@ static int check_syncdbs(alpm_list_t *dbnames)
 		pkglist = alpm_db_get_pkgcache(db);
 		syncpkglist = alpm_list_join(syncpkglist, alpm_list_copy(pkglist));
 	}
-	ret += checkdeps(syncpkglist);
+	ret += check_deps(syncpkglist);
 
 cleanup:
 	alpm_list_free(syncpkglist);
@@ -174,18 +243,18 @@ cleanup:
 
 static void usage(void)
 {
-	fprintf(stderr, "usage:\n");
-	fprintf(stderr,
-			"\t%s [-b <pacman db>]                : check the local database\n", BASENAME);
-	fprintf(stderr,
-			"\t%s [-b <pacman db>] core extra ... : check the listed sync databases\n", BASENAME);
+	fprintf(stderr, "testdb (pacman) v" PACKAGE_VERSION "\n\n"
+			"Test a pacman local database for validity.\n\n"
+			"Usage: testdb [options]\n\n"
+			"  -b <pacman db>                : check the local database\n"
+			"  -b <pacman db> core extra ... : check the listed sync databases\n");
 	exit(1);
 }
 
 int main(int argc, char *argv[])
 {
-	int ret = 0;
-	enum _alpm_errno_t err;
+	int errors = 0;
+	alpm_errno_t err;
 	const char *dbpath = DBPATH;
 	int a = 1;
 	alpm_list_t *dbnames = NULL;
@@ -216,13 +285,13 @@ int main(int argc, char *argv[])
 	alpm_option_set_logcb(handle, output_cb);
 
 	if(!dbnames) {
-		ret = check_localdb();
+		errors = check_localdb();
 	} else {
-		ret = check_syncdbs(dbnames);
+		errors = check_syncdbs(dbnames);
 		alpm_list_free(dbnames);
 	}
 
-	cleanup(ret);
+	cleanup(errors > 0);
 }
 
-/* vim: set ts=2 sw=2 noet: */
+/* vim: set noet: */
